    # An Ampl version of Automatic voltage control
	# Get askparm1.dat file for parameter settings
	
   reset;
   
   
   param G {i in 1..16, j in 1..16}; #
   param B {i in 1..16, j in 1..16}; #
   
   param time; # current time
   param totaltime;
   
   
   param run; # current run
   
   param case; # current case called from bash command
   
   param Qcap {i in 1..16}; #pi equivalent cap
     
   param Qprev {i in 1..16};
   param Vref {i in 1..16}; # Nominal Voltage
   
   param LQ{i in 1..16}; # Limits for number of Reactors
   param UQ{i in 1..16}; # Limits for number of capacitor banks
   param Lsvc{i in 1..16}; # Limits SVC
   param Usvc{i in 1..16}; # Limits SVC 
   param save; #ensures smooth objective function
   
   param Vmin; # Nominal Voltage min
   param Vmax; # Nominal Voltage max
   
   
   param Pdem {i in 1..16,j in 1..totaltime}; # Real power demand
     
   
   
   param wV; # Weight voltage
   param wQ;
   #param wdiff; # Weight for switching both capacitors and reactors, now: for svc use  
   param wsvc; # Weight for svc use     
   
   
   # param wSVC; # Weight SVC
   
   var P{1..16,1..totaltime};
   
   # Q limits
   var Q{i in 1..16} integer >= LQ[i], <= UQ[i];
   var Qsvc{i in 1..16}  >=Lsvc[i], <= Usvc[i];
   var V{1..16} >=0;
   var delta{1..16} >=-3.14, <=3.14;
   var e{1..16};
   var d{1..16};
   var Qswitch{1..16};
#    var pos{1..16} binary;
#    var neg{1..16} binary;
   
    minimize cost:
       wV * (sum{i in 1..16}(e[i]^2)) + wsvc * sum{i in 1..16}((0-Qsvc[i])^2) + 0.001 * sum{i in 1..16} Q[i]^2; # + wQ[time] * sum{i in 1..16}(Qswitch[i]^2); #+wdiff * ((sum{i in 1..16}(neg[i])) * (sum{i in 1..16}(pos[i])));
       
       #*(-neg[i]+pos[i]) #used for absolute value calculations
       
       #wV * (sum{i in 1..16}(e[i]^2)) + wQ * (sum{i in 1..16}(abs(Qswitch[i])));
       # + wSVC * sum((mprev(j) - m(j))^2);
       

   subject to
       
    voltageErrorMin{i in 1..16}: -e[i]+Vmin <=V[i];
    voltageErrorMax{i in 1..16}: e[i]+Vmax >=V[i];
   
    Qcalc{i in 1..16}: Qswitch[i]=Qprev[i] - Q[i];
    
    #Absolutt value calculations, unnessesary if absolute values are removed from the cost function
    
# 	IsXPositive{i in 1..16}: Qswitch[i] <= UQ*pos[i] - save*neg[i];
# 	IsXNegative{i in 1..16}: Qswitch[i] >= LQ*neg[i] + save*pos[i]; 
# 	Picksign{i in 1..16}:neg[i]+pos[i]<=1;
       
    # Power demands
    c1{i in 1..totaltime}: P[1,i]=Pdem[1,i]; # 9 and 16 are the surplus nodes   
    c2{i in 1..totaltime}: P[2,i]=Pdem[2,i];
    c3{i in 1..totaltime}: P[3,i]=Pdem[3,i];
    c4{i in 1..totaltime}: P[4,i]=Pdem[4,i];
    c5{i in 1..totaltime}: P[5,i]=Pdem[5,i];
    c6{i in 1..totaltime}: P[6,i]=Pdem[6,i];
  	c7{i in 1..totaltime}: P[7,i]=Pdem[7,i];
  	c8{i in 1..totaltime}: P[8,i]=Pdem[8,i];
  	c10{i in 1..totaltime}: P[10,i]=Pdem[10,i];      		
  	c11{i in 1..totaltime}: P[11,i]=Pdem[11,i];      		
    c12{i in 1..totaltime}: P[12,i]=Pdem[12,i];
    c13{i in 1..totaltime}: P[13,i]=Pdem[13,i]; 
    c14{i in 1..totaltime}: P[14,i]=Pdem[14,i];     		
    c15{i in 1..totaltime}: P[15,i]=Pdem[15,i]; 
    
    
   /* for{i in 2..totaltime}{ #all commented out
	    drop c1[i];
	   	drop c2[i];
	    drop c3[i];
	    drop c4[i];
	    drop c5[i];
	    drop c6[i];
	    drop c7[i];
	    drop c8[i];
	    drop c10[i];
	    drop c11[i];
	    drop c12[i];
	    drop c13[i];
	    drop c14[i];
	    drop c15[i];
    }*/
    
    

    #init power flow
    cV: V[16]=1;
    cdelta: delta[16]=0;
    c9{i in 1..totaltime}: P[9,i] >= 0;
    c16{i in 1..totaltime}: P[16,i] >= 0;
       
    realpower{i in 1..16}: P[i,time]=sum{k in 1..16} (V[i]*V[k]*(G[i,k]*cos(delta[i]-delta[k]) + B[i,k]*sin(delta[i]-delta[k]))); #real power flow
  
    reactivepower{i in 1..16}: (Q[i]+Qcap[i]+Qsvc[i])=sum{k in 1..16} (V[i]*V[k]*(G[i,k]*sin(delta[i]-delta[k]) - B[i,k]*cos(delta[i]-delta[k]))); # reactive power flow equations
  	 	
    #calctotal: total=sum{k in 1..16} Qswitch[k];
   
    data ("setcase.dat"); 
	data ("param_decent/askparam"& case &".dat");

   option solver bonmin; # Choose BONMIN as the solver (assuming that
                         # bonmin is in your PATH
   option presolve 0;         
              
   options bonmin_options "bonmin.allowable_gap 0.00099 bonmin.time_limit 900 bonmin.nlp_log_level 0 bonmin.num_resolve_at_root 10 max_iter=100"; #options set both ipopt-options and bonmin-option, for bonmin-options use bonmin. as a prefix
   
   #bonmin.cutoff 5000000
   
   # bonmin.bb_log_level 0 bonmin.bb_log_interval 0
                         
   option log_file ("logg_cont_decent/run"&run&"/asklog"&run&time&".log");
                         
                         
   solve;                # Solve the model
   
   print cost > ("logg_cont_decent/run"&run&"/cost"&run&time&".log");
   #print cost.bestbound > ("logg_cont_decent/run"&run&"/costbestbound"&run&time&".log");
   print{i in 1..16} V[i] > ("logg_cont_decent/run"&run&"/voltage"&run&time&".log");
   print{i in 1..16} delta[i] > ("logg_cont_decent/run"&run&"/delta"&run&time&".log");
   print{i in 1..16} P[i,1] > ("logg_cont_decent/run"&run&"/power"&run&time&".log");
   print{i in 1..16} Q[i] > ("logg_cont_decent/run"&run&"/reactive"&run&time&".log");
   print{i in 1..16} Qprev[i] > ("logg_cont_decent/run"&run&"/prev"&run&time&".log");
   print{i in 1..16} Qswitch[i] > ("logg_cont_decent/run"&run&"/switch"&run&time&".log");
   print{i in 1..16} Qsvc[i] > ("logg_cont_decent/run"&run&"/svc"&run&time&".log");
   print _total_solve_elapsed_time > ("logg_cont_decent/run"&run&"/time"&run&time&".log");
   
   
   display case;
   display time;

   
   
   # Rerun the problem for multiple time instances
   
   
   delete minimize cost;  #use for including cost of switching
   redeclare minimize cost: wV * (sum{i in 1..16}(e[i]^2)) + wQ * sum{i in 1..16}(Qswitch[i]^2) + wsvc * sum{i in 1..16}((0-Qsvc[i])^2) + 0.001 * sum{i in 1..16} Q[i]^2; 
   
   for {n in 2..totaltime}	{
   
   #Setting power demands for new hour
   
      drop c1[n-1];
	  drop c2[n-1];
	  drop c3[n-1];
	  drop c4[n-1];
	  drop c5[n-1];
	  drop c6[n-1];
	  drop c7[n-1];
	  drop c8[n-1];
	  drop c10[n-1];
	  drop c11[n-1];
	  drop c12[n-1];
	  drop c13[n-1];
	  drop c14[n-1];
	  drop c15[n-1];
	  
	  
   
   
   reset data Qprev,time;
   
   let time:=n;
   

   #let wQ:=0.001;
  
   
   let Qprev[1] := Q[1];
   let Qprev[2] := Q[2];
   let Qprev[3] := Q[3];
   let Qprev[4] := Q[4];
   let Qprev[5] := Q[5];
   let Qprev[6] := Q[6];
   let Qprev[7] := Q[7];
   let Qprev[8] := Q[8];
   let Qprev[9] := Q[9];
   let Qprev[10] := Q[10];
   let Qprev[11] := Q[11];
   let Qprev[12] := Q[12];
   let Qprev[13] := Q[13];
   let Qprev[14] := Q[14];
   let Qprev[15] := Q[15];
   let Qprev[16] := Q[16];
   
   
   
   option solver bonmin; # Choose BONMIN as the solver (assuming that
                         # bonmin is in your PATH
            
   
   options bonmin_options "bonmin.allowable_gap 0.0009999 bonmin.time_limit 300 bonmin.nlp_log_level 0 bonmin.num_resolve_at_root 10 max_iter=100" ; 
   
    #bonmin.cutoff 5000000
   
   option log_file ("logg_cont_decent/run"&run&"/asklog"&run&time&".log");
   
#    	option times 1;
# 	option gentimes 1;
# 	option show_stats 1;
	option presolve 0;
                         
                         
   solve;                # Solve the model
   
   print cost > ("logg_cont_decent/run"&run&"/cost"&run&time&".log");
   #print cost.bestbound > ("logg_cont_decent/run"&run&"/costbestbound"&run&time&".log");
   print{j in 1..16} V[j] > ("logg_cont_decent/run"&run&"/voltage"&run&time&".log");
   print{j in 1..16} delta[j] > ("logg_cont_decent/run"&run&"/delta"&run&time&".log");
   print{j in 1..16} P[j,time] > ("logg_cont_decent/run"&run&"/power"&run&time&".log");
   print{j in 1..16} Q[j] > ("logg_cont_decent/run"&run&"/reactive"&run&time&".log");
   print{j in 1..16} Qprev[j] > ("logg_cont_decent/run"&run&"/prev"&run&time&".log");
   print{j in 1..16} Qswitch[j] > ("logg_cont_decent/run"&run&"/switch"&run&time&".log");
   print{j in 1..16} Qsvc[j] > ("logg_cont_decent/run"&run&"/svc"&run&time&".log");
   print _total_solve_elapsed_time > ("logg_cont_decent/run"&run&"/time"&run&time&".log");
   
   display case;
   display time;
   }