%Specialization Project, Modeling of the Southern Norwegian Power grid
% clear;
% close all;
% clc;
Y=zeros(16,16); % impedance per node
linechar_420V=0.02+0.32j; %line characteristics for 420 V duplex in Ohm/km
cablechar_420V=1+1j; %Cable characteristics for 420 V underwater
f=50; %Frequency of Power system [Hz = 1/s]
V_base=410e3; %Per unit [pu] based on 410 kV
S_base=100e6; %Per unit [pu] based on 50 MVA
Z_base=V_base^2/S_base;
z_420=(abs(linechar_420V)*exp(1j*angle(linechar_420V)))/Z_base; %same as linechar_420V
omega=2*pi*f;
nodes=16;
edges=19;
time=1; %keeps time in AMPL model
totaltime=12; %number of time steps the model is run

%Line length, per line (from google maps (straight line, crow kilometers))
l(1)=70;%line length [km]
l(2)=177;%line length [km]
l(3)=25;%line length [km]
l(4)=73;%line length [km]
l(5)=22;%line length [km]
l(6)=215;%line length [km]
l(7)=84;%line length [km]
l(8)=132;%line length [km]
l(9)=196;%line length [km]
l(10)=49;%line length [km]
l(11)=187;%line length [km]
l(12)=77;%line length [km]
l(13)=63;%line length [km]
l(14)=120;%line length [km]
l(15)=92;%line length [km]
l(16)=20;%line length [km]
l(17)=49;%line length [km]
l(18)=44;%line length [km]
l(19)=55;%line length [km]
l=l';
l=l;

% Line length, per line (from google maps (straight line, crow kilometers))
% l(1)=7;%line length [km]
% l(2)=17;%line length [km]
% l(3)=2;%line length [km]
% l(4)=7;%line length [km]
% l(5)=2;%line length [km]
% l(6)=21;%line length [km]
% l(7)=8;%line length [km]
% l(8)=13;%line length [km]
% l(9)=19;%line length [km]
% l(10)=4;%line length [km]
% l(11)=18;%line length [km]
% l(12)=7;%line length [km]
% l(13)=6;%line length [km]
% l(14)=12;%line length [km]
% l(15)=9;%line length [km]
% l(16)=2;%line length [km]
% l(17)=4;%line length [km]
% l(18)=4;%line length [km]
% l(19)=5;%line length [km]
% l=l';


%Capacitans on pi-equivalent (half in each end) per line
cap=11.4e-9/(2*Z_base); %[nF/km]


%Capasitance modeled as impedance per line, on each side (therefore divided by 2)
for i=1:edges
    X_c(i)=1/(1j*omega*cap*l(i));
end

X_c(1:19)=inf;%X_c*(1/Z_base);

Q_base=(518.6e3)/S_base;

for i=1:edges
   Q_cape(i)=Q_base*l(i);
end

Q_cap(1)=Q_cape(1)+Q_cape(19);
Q_cap(2)=Q_cape(1)+Q_cape(2);
Q_cap(3)=Q_cape(2)+Q_cape(3)+Q_cape(7);
Q_cap(4)=Q_cape(3)+Q_cape(4);
Q_cap(5)=Q_cape(4)+Q_cape(5)+Q_cape(9);
Q_cap(6)=Q_cape(5)+Q_cape(6)+Q_cape(7)+Q_cape(11);
Q_cap(7)=Q_cape(9)+Q_cape(10);
Q_cap(8)=Q_cape(8)+Q_cape(10);
Q_cap(9)=Q_cape(11)+Q_cape(12);
Q_cap(10)=Q_cape(8)+Q_cape(12)+Q_cape(13);
Q_cap(11)=Q_cape(13)+Q_cape(14);
Q_cap(12)=Q_cape(14)+Q_cape(15);
Q_cap(13)=Q_cape(15)+Q_cape(16);
Q_cap(14)=Q_cape(16)+Q_cape(17)+Q_cape(6);
Q_cap(15)=Q_cape(17)+Q_cape(18);
Q_cap(16)=Q_cape(18)+Q_cape(19);


%Reactive components
X_battery=1/(1j*omega*0.0032);
X_reactor=1j*omega*0.0032;
Y_battery=-1/X_battery;
Y_reactor=-1/X_reactor;

%Series impedance per line
for i=1:edges
    y(i)=1/(z_420*l(i));
end


% y(1)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(2)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(3)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(4)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(5)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(6)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(7)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(8)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(9)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(10)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(11)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(12)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(13)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(14)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(15)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(16)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(17)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(17)=1/z_420;%impedance per km [(1/Ohm)/km]
% y(18)=1/z_420;%impedance per km [(1/Ohm)/km]
% y=y';


% Network equations
Y(1,2)=-y(1);
Y(2,1)=Y(1,2);

Y(2,3)=-y(2);
Y(3,2)=Y(2,3);

Y(3,4)=-y(3);
Y(4,3)=Y(3,4);

Y(4,5)=-y(4);
Y(5,4)=Y(4,5);

Y(5,6)=-y(5);
Y(6,5)=Y(5,6);

Y(6,14)=-y(6);
Y(14,6)=Y(6,14);

Y(3,6)=-y(7);
Y(6,3)=Y(3,6);

Y(8,10)=-y(8);
Y(10,8)=Y(8,10);

Y(5,7)=-y(9);
Y(7,5)=Y(5,7);

Y(7,8)=-y(10);
Y(8,7)=Y(7,8);

Y(6,9)=-y(11);
Y(9,6)=Y(6,9);

Y(9,10)=-y(12);
Y(10,9)=Y(9,10);

Y(10,11)=-y(13);
Y(11,10)=Y(10,11);

Y(11,12)=-y(14);
Y(12,11)=Y(11,12);

Y(12,13)=-y(15);
Y(13,12)=Y(12,13);

Y(13,14)=-y(16);
Y(14,13)=Y(13,14);

Y(14,15)=-y(17);
Y(15,14)=Y(14,15);

Y(15,16)=-y(18);
Y(16,15)=Y(15,16);

Y(16,1)=-y(19);
Y(1,16)=Y(16,1);



%Nodal impedance based on branches
for i=1:nodes
    for k=1:nodes
        if i~=k
            Y(i,i)=Y(i,i)-Y(i,k);
        end
    end
end

%Nodal impedance for pi-equivalents and compensations

%Nodal impedance node 1
Y(1,1)=Y(1,1);%+1/(X_c(1)+X_c(19));
%Shunt capacitors
% if t>12
%     n_1=1; %number of components
% else
    n_1=0;
% end
Y(1,1)=Y(1,1)+n_1*Y_battery;
%Shunt reactors
% if t>12
%     m_1=1; %number of components
% else
m_1=0; %number of components
% end
Y(1,1)=Y(1,1)+m_1*Y_reactor;

%Nodal impedance node 2
Y(2,2)=Y(2,2);%+1/(X_c(1)+X_c(2));
%Shunt capacitors
n_2=0; %number of components
Y(2,2)=Y(2,2)+n_2*Y_battery;
%Shunt reactors
m_2=0; %number of components
Y(2,2)=Y(2,2)+m_2*Y_reactor;

%Nodal impedance node 3
Y(3,3)=Y(3,3);%+1/(X_c(2)+X_c(3)+X_c(7));
%Shunt capacitors
n_3=0; %number of components
Y(3,3)=Y(3,3)+n_3*Y_battery;
%Shunt reactors
m_3=0; %number of components
Y(3,3)=Y(3,3)+m_3*Y_reactor;

%Nodal impedance node 4
Y(4,4)=Y(4,4);%+1/(X_c(3)+X_c(4));
%Shunt capacitors
n_4=0; %number of components
Y(4,4)=Y(4,4)+n_4*Y_battery;
%Shunt reactors
m_4=0; %number of components
Y(4,4)=Y(4,4)+m_4*Y_reactor;

%Nodal impedance node 5
Y(5,5)=Y(5,5);%+1/(X_c(4)+X_c(5)+X_c(9));
%Shunt capacitors
n_5=0; %number of components
Y(5,5)=Y(5,5)+n_5*Y_battery;
%Shunt reactors
m_5=0; %number of components
Y(5,5)=Y(5,5)+m_5*Y_reactor;

%Nodal impedance node 6
Y(6,6)=Y(6,6);%+1/(X_c(5)+X_c(6)+X_c(7)+X_c(11));
%Shunt capacitors
n_6=0; %number of components
Y(6,6)=Y(6,6);%+n_6*Y_battery;
%Shunt reactors
m_6=0; %number of components
Y(6,6)=Y(6,6)+m_6*Y_reactor;

%Nodal impedance node 7
Y(7,7)=Y(7,7);%+1/(X_c(9)+X_c(10));
%Shunt capacitors
n_7=0; %number of components
Y(7,7)=Y(7,7)+n_7*Y_battery;
%Shunt reactors
m_7=0; %number of components
Y(7,7)=Y(7,7)+m_7*Y_reactor;

%Nodal impedance node 8
Y(8,8)=Y(8,8);%+1/(X_c(8)+X_c(10));
%Shunt capacitors
n_8=0; %number of components
Y(8,8)=Y(8,8)+n_8*Y_battery;
%Shunt reactors
m_8=0; %number of components
Y(8,8)=Y(8,8)+m_8*Y_reactor;

%Nodal impedance node 9
Y(9,9)=Y(9,9);%+1/(X_c(11)+X_c(12));
%Shunt capacitors
n_9=0; %number of components
Y(9,9)=Y(9,9)+n_9*Y_battery;
%Shunt reactors
m_9=0; %number of components
Y(9,9)=Y(9,9)+m_9*Y_reactor;

%Nodal impedance node 10
Y(10,10)=Y(10,10);%+1/(X_c(8)+X_c(12)+X_c(13));
%Shunt capacitors
n_10=0; %number of components
Y(10,10)=Y(10,10)+n_10*Y_battery;
%Shunt reactors
m_10=0; %number of components
Y(10,10)=Y(10,10)+m_10*Y_reactor;

%Nodal impedance node 11
Y(11,11)=Y(11,11);%+1/(X_c(13)+X_c(14));
%Shunt capacitors
n_11=0; %number of components
Y(11,11)=Y(11,11)+n_11*Y_battery;
%Shunt reactors
m_11=0; %number of components
Y(11,11)=Y(11,11)+m_11*Y_reactor;

%Nodal impedance node 12
Y(12,12)=Y(12,12);%+1/(X_c(14)+X_c(15));
%Shunt capacitors
n_12=0; %number of components
Y(12,12)=Y(12,12)+n_12*Y_battery;
%Shunt reactors
m_12=0; %number of components
Y(12,12)=Y(12,12)+m_12*Y_reactor;

%Nodal impedance node 13
Y(13,13)=Y(13,13);%+1/(X_c(15)+X_c(16));
%Shunt capacitors
n_13=0; %number of components
Y(13,13)=Y(13,13)+n_13*Y_battery;
%Shunt reactors
m_13=0; %number of components
Y(13,13)=Y(13,13)+m_13*Y_reactor;

%Nodal impedance node 14
Y(14,14)=Y(14,14);%+1/(X_c(6)+X_c(16)+X_c(17));
%Shunt capacitors
n_14=0; %number of components
Y(14,14)=Y(14,14)+n_14*Y_battery;
%Shunt reactors
m_14=0; %number of components
Y(14,14)=Y(14,14)+m_14*Y_reactor;

%Nodal impedance node 15
Y(15,15)=Y(15,15);%+1/(X_c(17)+X_c(18));
%Shunt capacitors
n_15=0; %number of components
Y(15,15)=Y(15,15)+n_15*Y_battery;
%Shunt reactors
m_15=0; %number of components
Y(15,15)=Y(15,15)+m_15*Y_reactor;

%Nodal impedance node 16
Y(16,16)=Y(16,16);%+1/(X_c(6)+X_c(18)+X_c(19));
%Shunt capacitors
n_16=0; %number of components
Y(16,16)=Y(16,16)+n_16*Y_battery;
%Shunt reactors
m_16=0; %number of components
Y(16,16)=Y(16,16)+m_16*Y_reactor;

G=real(Y);
B=imag(Y);
theta=angle(Y);
% 
% % Define type of bus, slack bus(1), load bus(2) or Voltage controlled
% % bus(3)
% type(16)=1;
% type(1:15)=2;
% type(7)=3;
% type(9)=3;
% type(11)=3;  
% type(14)=3;

% Initiate power flow equations and optimazation

%References
V_ref(1:16)=1;
Q_prev(1:16)=0;
V(1:16)=1;
delta(1:16)=0;

V_min=400e3/V_base;
V_max=420e3/V_base;

% Weights
w_V=1000000;
w_Q=0.1; %to be changed in ampl file
w_svc=1;
%w_diff=0.01;

% Number of reactive components
L_Q=-5;
U_Q=5;
L_svc=-2;
U_svc=2;
save=0.001; % gives smooth cost function for Q=0;

dem=zeros(16,totaltime);

% Get power demands

light=xlsread('power_demand_light.xlsx',1,'C4:R15');
 
dem=light';


run=1;




Pdem=(dem*1e6)/S_base;


    
    


Q(1:16)=0;


runstring=num2str(run);

parameterfile=strcat('param_light2/askparam',runstring,'.dat');

fid=fopen(parameterfile,'w');
fprintAmplParam(fid,'G',G);
fprintAmplParam(fid,'B',B);
fprintAmplParam(fid,'time',time);
fprintAmplParam(fid,'Pdem',Pdem);
fprintAmplParam(fid,'Qcap',Q_cap);
fprintAmplVar(fid,'P',Pdem);
fprintAmplVar(fid,'Q',Q);
% fprintAmplParam(fid,'Vref',V_ref);
fprintAmplParam(fid,'Qprev',Q_prev);
fprintAmplVar(fid,'V',V);
fprintAmplVar(fid,'delta',delta);
fprintAmplParam(fid,'wV',w_V);
fprintAmplParam(fid,'wQ',w_Q);
fprintAmplParam(fid,'wsvc',w_svc);
% fprintAmplParam(fid,'wdiff',w_diff);
fprintAmplParam(fid,'Vmin',V_min);
fprintAmplParam(fid,'Vmax',V_max);
fprintAmplParam(fid,'LQ',L_Q);
fprintAmplParam(fid,'UQ',U_Q);
fprintAmplParam(fid,'Lsvc',L_svc);
fprintAmplParam(fid,'Usvc',U_svc);
fprintAmplParam(fid,'save',save);
fprintAmplParam(fid,'run',run);
fprintAmplParam(fid,'totaltime',totaltime);
fclose(fid);





%Nodes - "P" indicate produces:
%1: Feda - P, Q
%2: Kristiansand, - P, Q
%3: Rd, - P, Q
%4: Hasle - P, Q
%5: Frogner - P, Q
%6: Sylling - P, Q
%P(7): vre Vinstra -  P, Q
%8: Vgmo - P, Q
%P(9): Sima - P, Q
%10: Fardal - P, Q
%P(11): Refsdal - P, Q
%12: Samnanger - P, Q
%13: Sauda - P, Q
%P(14): Saurdal - P, Q
%15: Lyse - P, Q
%P(16): Tonstad - Referance node/slackbus V=1 pu, delta=0




%Connenctions:
%1: 1-2 70 km
%2: 2-3 177 km
%3: 3-4 25 km underwater
%4: 4-5 73 km
%5: 5-6 22 km
%6: 6-14 215 km
%7: 3-6 84 km
%8: 8-10 132 km
%9: 5-7 196 km
%10: 7-8 49 km 
%11: 6-9 187 km
%12: 9-10 77 km
%13: 10-11 63 km
%14: 11-12 120 km
%15: 12-13 92 km
%16: 13-14 20 km
%17: 14-15  49 km
%18: 15-16 44 km
%19: 16-1 45 km

%Totalt +-1000-4000 MW lastflyt mellom noder s. 39 i Kraftsystem utredning